
/* gvfs.h -- The virtual filesystem layer interface.

 * Copyright (C) 1999 Steve Hill <sjhill@plutonium.net>
 *                    Yarick Rastrigin <yarick@relex.ru>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __GVFS_H__
#define __GVFS_H__

#include <sys/stat.h>

#include "gvfs_ftp.h"
#include "gvfs_local.h"
#include "gvfs_smb.h"

typedef struct _GVFS GVFS;
typedef gboolean (*FSItemTypeCheckFunc) (void *item_stat_info);
typedef struct _gvfs_file_copy_struct gvfs_file_copy_struct;
struct _GVFS
  {
    GVFS *next;
    gboolean read_only;
      gint (*init) (GVFS * gvfs);
    void *(*opendir) (gchar * dirname);
    void *(*readdir) (void *gvfs_info);
      gint (*closedir) (void *gvfs_info);
      gint (*stat) (gchar * path, struct stat * buf);
      gint (*lstat) (gchar * path, struct stat * buf);
      gint (*readlink) (gchar * path, void *buf, gint size);
      gint (*unlink) (gchar * filename);
    void *(*fopen) (gchar * filename, gchar * mode);
      gint (*fclose) (void *fd);
      gint (*fread) (void *fd, void *buffer, gulong size);
      gint (*fwrite) (void *fd, void *buffer, gulong size);
      gint (*mkdir) (gchar * filename);
      gint (*rm) (gchar * flename);
      gint (*rmdir) (gchar * filename);
      gint (*symlink) (gchar * symlink_name, gchar * symlink_target);
      gint (*mv) (gchar * from, gchar * to);
      gint (*chmod) (gchar * filename, int mode);
      gint (*chown) (gchar * filename, int owner, int group);
  };

struct _gvfs_file_copy_struct
  {
    gchar *source_name;
    gchar *target_name;
    void *source_fd;
    void *target_fd;
    void *buffer;
    int owner, group, mode;
    gboolean force_overwrite;
    gboolean all_ok;		/* after iserting this var I'm feeling this structure is seriously overloaded :) */
    gulong total_bytes;
    gulong done_bytes;
    GVFS *source_gvfs;
    GVFS *target_gvfs;
  };

/* Other filesystems. */
extern GVFS gvfs_ftp_fs_ops;
extern GVFS gvfs_local_fs_ops;
extern GVFS gvfs_smb_fs_ops;

/* Initializes the virtual filesystem layer. */
guint gvfs_initialize (void);

/* Shutdown the virtual filesystem layer. */
void gvfs_shutdown (void);

/* Returns the current directory. */
gchar *gvfs_get_current_dir (void);

/* Sets the current directory */
void gvfs_set_current_dir (gchar * directory);

/* Determine which filesystem handler to use. */
GVFS *gvfs_type (gchar * path);

/* 'opendir' wrapper function. */
void *gvfs_opendir (GVFS * gvfs, gchar * dirname);

/* 'readdir' wrapper function. */
void *gvfs_readdir (GVFS * gvfs, void *dir_info);

/* 'closedir' wrapper function. */
gint gvfs_closedir (GVFS * gvfs, void *dir_info);

/* 'stat' wrapper function. */
gint gvfs_stat (GVFS * gvfs, gchar * filename, void *buf);

/* Symlink stats */
gint gvfs_lstat (GVFS * gvfs, gchar * filename, void *buf);


/* 'unlink' wrapper function. */
gint gvfs_unlink (gchar * filename);

/* 'mkdir()' wrapper */
gint gvfs_mkdir (gchar * path);

/* symlink creation wrapper */
gint gvfs_symlink (gchar * symlink_name, gchar * symlink_target);

/* symlink info wrapper */
gint gvfs_readlink (gchar * path, void *buf, gint bufsize);

/* One-file low-level copying operation */
gint
gvfs_copy_file (gvfs_file_copy_struct * gfcs);

/*
   The main idea is to make file copying operation as transparent for caller and independant of
   sources and destinations as possible.
   So we're building some struct with source and target names (like:
   source_name="ftp://relex.ru/home/yarick/homepage/index.html"
   target_name = /home/yarick/HomePage/indx.html
   source_gvfs = gvfs_type(source_name); / Must point to gvfs_ftp/
   target_gvfs = gvfs_type(target_gvfs); / Points to gvfs_local  /
   source_fd = (*source_gvfs->fopen)(source_name,"r");
   target_fd =  (*target_gvfs->fopen)(target_name,"w");
   total_bytes = earlier retrieved value;
   done_bytes = 0;
   ) in gvfs_init_fcs
   and then doing atomic copying of file via
   source_gvfs->read(source_fd, somebuffer, howmanybytes); with source_fd which could be a
   FILE * structure or socket or aanything else protocol depending.
   target_gvfs->write(target_fd, somebuffer, howmanybytes);
   After every operation we're returning results to caller for possible errors handling
   and progress update.
   After we're completely done we're closing fd's and cleaning memory in gvfs_shut_fcs
   )
 */

/* Initialiser for file copy structure */
gint
gvfs_init_fcs (gvfs_file_copy_struct * gfcs);

/* And finaliser... */
gint
gvfs_shut_fcs (gvfs_file_copy_struct * gfcs);

/* Self-explanatory :) */
gint gvfs_rm (gchar * filename);
gint gvfs_rmdir (gchar * filename);
gint gvfs_rename (gchar * from, gchar * to);
gint gvfs_chmod (gchar * filename, int mode);
gint gvfs_chown (gchar * filename, int owner, int group);
/* Check to see if a directory has children. */
gboolean gvfs_has_subdirs (gchar * path);


/* Returns a list of possibly sorted dir entries excluding "." and ".." entries. */
GList *gvfs_get_x_list (gchar * path, FSItemTypeCheckFunc check_func, gboolean need_sort, GList ** stat_info);

#endif /* __GVFS_H__ */
